#!/usr/bin/python
#
# watch-composes.py - A utility to watch a set of compose trees for changes
#       and to kick off tests if the compose changes.
#
# Copyright (c) 2009 Red Hat
#
# Authors:
#     Jesse Keating <jkeating@redhat.com>

import urlgrabber
import os
import sys
import subprocess
from autoqa.repoinfo import repoinfo
import optparse

# Set up the option parser
parser = optparse.OptionParser(description='A utility to watch a set of \
compose trees for changes and to kick off tests if the compose changes.')
parser.add_option('--dryrun', '--dry-run', action='store_true',
    help='Do not actually execute commands, just show what would be done')
(opts, args) = parser.parse_args()

# Set the default arch to our placeholder, '%a'
repoinfo.setarch('%%a') # two %% because of ConfigParser interpolation

# Define a list of repos to watch, by name (see /etc/repoinfo.conf)
watchcomposes = ['rawhide']

# Setup a cache path
cachedir = '/var/cache/autoqa/watch-composes'
if not os.path.isdir(cachedir):
    os.makedirs(cachedir)

# Setup an empty dict of testable repos
testable = {}
repodict = {} # a place to save repo info as read

# Cycle through our list of repo watches and determine which should be tested.
for reponame in watchcomposes:
    repo = repoinfo.getrepo(reponame)
    repodict[reponame] = repo
    if not reponame in testable:
        testable[reponame] = []
    for arch in repo['arches']:
        try:
            cachefile = os.path.join(cachedir,repo['path'], arch, 'treeinfo')
            cache = open(cachefile).read()
        except IOError:
            # If we don't have a cache file, assign cache to blank.  This will
            # let us still try to hit the .treeinfo and make sure the repo is
            # alive.
            cache = ''
        # check .treeinfo
        try:
            # XXX use urljoin?
            urlpath = os.path.join(repo['url'].replace('%a',arch),
                                   '.treeinfo')
            remote = urlgrabber.urlread(urlpath)
        # XXX maybe less broad exception?
        except Exception, e:
            print "Error from %s: %s" % (urlpath, e)
            continue
        # Now compare the remote to the cache
        if cache != remote:
            testable[reponame].append(arch)

# Cycle through the testable repos and update the cache and launch harness
for reponame, arches in testable.items():
    if not arches:
        continue
    repo = repodict[reponame]
    # construct arguments for call to test launcher
    harnesscall = ['autoqa', 'post-tree-compose', '--name', repo['name']]
    for arch in arches:
        harnesscall += ['--arch', arch]
    harnesscall.append(repo['url'])
    if opts.dryrun:
        print ' '.join(harnesscall)
        continue

    # update cache
    for arch in arches:
        cachepath = os.path.join(cachedir, repo['path'], arch)
        if not os.path.exists(cachepath):
            os.makedirs(cachepath)
        composepath = repo['url'].replace('%a', arch)
        urlpath = os.path.join(composepath, '.treeinfo')
        output = os.path.join(cachepath, 'treeinfo')
        urlgrabber.urlgrab(urlpath, filename=output)

    # launch tests
    subprocess.call(harnesscall)
