#
# repoinfo.py - get repository info from repoinfo.cfg for autotest watchers
#
# Copyright 2009, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Author: Will Woods <wwoods@redhat.com>

from ConfigParser import SafeConfigParser
import warnings

configpath = ('/etc/autoqa/repoinfo.conf',
              '/etc/repoinfo.conf',
              'repoinfo.conf')

class RepoinfoConfig(object):
    def __init__(self, arch="$basearch", filelist=configpath, defaults={}):
        self.defaults = defaults
        self.filelist = filelist
        self.setarch(arch) # implies read()

    def _getlist(self, name, item):
        itemlist = self.config.get(name, item)
        if not itemlist:
            return []
        else:
            return [t.strip() for t in itemlist.split(',')]
    def getarches(self, name):
        return self._getlist(name, 'arches')
    def getparents(self, name):
        return self._getlist(name, 'parents')
    def getrepo(self, name):
        repo = {'arches': self.getarches(name),
                'parents': self.getparents(name),
                'tag': self.config.get(name,'tag'),
                'url': self.config.get(name,'url'),
                'path': self.config.get(name,'path'),
                'name': name # kind of unnecessary
        }
        return repo
    def getrepo_by_tag(self, tag):
        for r in self.repos():
            if self.config.get(r,'tag') == tag:
                return self.getrepo(r)
    def repos(self):
        return self.config.sections()
    def setarch(self, arch):
        self.defaults['arch'] = arch
        self.read()
    def read(self):
        self.config = SafeConfigParser(self.defaults)
        return self.config.read(self.filelist)
    def get(self, repo, key):
        return self.config.get(repo, key)

repoinfo = RepoinfoConfig()
if not repoinfo.repos():
    warnings.warn("No repos found in (%s)" % ', '.join(configpath),
                  RuntimeWarning)

if __name__ == '__main__':
    import pprint
    rd = dict([(n, repoinfo.getrepo(n)) for n in repoinfo.repos()])
    pprint.pprint(rd)
